/*
jGuard is a security framework based on top of jaas (java authentication and authorization security).
it is written for web applications, to resolve simply, access control problems.

http://sourceforge.net/projects/jguard/

Copyright (C) 2004  Charles GAY

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


jGuard project home page:
http://sourceforge.net/projects/jguard/

*/
package net.sf.jguard.ext;


import java.security.Policy;
import java.util.HashMap;
import java.util.Map;
import javax.security.auth.login.Configuration;
import net.sf.jguard.core.CoreConstants;
import net.sf.jguard.core.PolicyEnforcementPointOptions;
import net.sf.jguard.core.authentication.AccessContext;
import net.sf.jguard.core.authentication.bindings.AbstractAuthenticationBindingsFactory;
import net.sf.jguard.core.authentication.bindings.AuthenticationBindings;
import net.sf.jguard.core.authentication.bindings.AuthenticationBindingsFactory;
import net.sf.jguard.core.authentication.configuration.ConfigurationHelper;
import net.sf.jguard.core.authentication.configuration.JGuardConfiguration;
import net.sf.jguard.core.authentication.manager.AuthenticationManagerFactory;
import net.sf.jguard.core.authorization.AuthorizationHelper;
import net.sf.jguard.core.authorization.manager.AuthorizationManager;
import net.sf.jguard.core.authorization.policy.AccessControllerUtils;
import net.sf.jguard.core.authorization.policy.MultipleAppPolicy;
import net.sf.jguard.core.authorization.policy.PolicyHelper;
import net.sf.jguard.ext.authentication.jmx.JMXHelper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Utility class to install Authentication and Authorization  for an application.
 * @author <a href="mailto:diabolo512@users.sourceforge.net">Charles Gay</a>
 */
public class InitHelper {

    private static final Logger logger = LoggerFactory.getLogger(InitHelper.class.getName());

     private static AccessControllerUtils getAccessControllerUtils(String authorizationScope, MultipleAppPolicy policy) {
        AccessControllerUtils accessControllerUtils;
        if (CoreConstants.JVM_SCOPE.equalsIgnoreCase(authorizationScope)) {
            accessControllerUtils = new AccessControllerUtils();
        } else {
            accessControllerUtils = new AccessControllerUtils(policy);
        }
        return accessControllerUtils;
    }

    private static String getAuthorizationScope(Map authorizationSettings) {
        //handle authorization scope
        String authorizationScope = null;
        if (authorizationSettings.get(CoreConstants.SCOPE) != null) {
            authorizationScope = (String) authorizationSettings.get(CoreConstants.SCOPE);
        } else {
            authorizationScope = CoreConstants.LOCAL_SCOPE;
        }
        return authorizationScope;
    }

    private static AccessControllerUtils initAuthorization(AuthenticationBindings authBindings, String authorizationScope,Map  authorizationSettings, String applicationName) {

        authBindings.setApplicationAttribute(SecurityConstants.AUTHORIZATION_SCOPE, authorizationScope);
        AuthorizationManager authorizationManager = AuthorizationHelper.initAuthorization(authorizationSettings, applicationName);
        MultipleAppPolicy policy = getPolicy(authorizationScope);
        AccessControllerUtils accessControllerUtils = getAccessControllerUtils(authorizationScope, policy);
        policy.registerPermissionProvider(Thread.currentThread().getContextClassLoader(), authorizationManager);
        authBindings.setApplicationAttribute(SecurityConstants.ACCESS_CONTROLLER_UTILS, accessControllerUtils);
        authBindings.setApplicationAttribute(CoreConstants.AUTHORIZATION_MANAGER, authorizationManager);
        return accessControllerUtils;
    }

  


    private static MultipleAppPolicy getPolicy(String authorizationScope){
        MultipleAppPolicy policy = null;
         if(CoreConstants.JVM_SCOPE.equalsIgnoreCase(authorizationScope)){
	        PolicyHelper.installPolicyOnJVM();
			//Register the new authorization manager with jguard policy provider
			policy = (MultipleAppPolicy)Policy.getPolicy();
	    }else{
        	policy = new MultipleAppPolicy();
        }
        return policy;
    }


	private static JGuardConfiguration initAuthentication(String authenticationConfigurationLocation, String webappHomePath, String applicationName) {
		logger.debug(" ### initializing authentication ### ");

		JGuardConfiguration jGuardConf = null;
		Map authenticationSettings = ConfigurationHelper.loadConfiguration(authenticationConfigurationLocation,webappHomePath);

		String authenticationScope = null;
		if(authenticationSettings.get(CoreConstants.SCOPE)!=null){
			authenticationScope = (String)authenticationSettings.get(CoreConstants.SCOPE);
                }

		if(CoreConstants.JVM_SCOPE.equalsIgnoreCase(authenticationScope)){
			jGuardConf = (JGuardConfiguration) Configuration.getConfiguration();
		}else{
			jGuardConf = new JGuardConfiguration(applicationName,authenticationConfigurationLocation,webappHomePath);
		}

		logger.debug(" ### authentication initialized ### ");

        return jGuardConf;
	}

    private static void initAuthentication(String authenticationConfigurationLocation, String webappHomePath, String applicationName, AuthenticationBindings authBindings, AccessControllerUtils accessControllerUtils) {
        JGuardConfiguration jguardConf = initAuthentication(authenticationConfigurationLocation, webappHomePath, applicationName);
        authBindings.setApplicationAttribute(CoreConstants.JGUARD_CONFIGURATION, jguardConf);
        authBindings.setApplicationAttribute(CoreConstants.AUTHENTICATION_MANAGER, AuthenticationManagerFactory.getAuthenticationManager());
        //JMX MBeanServer creation
        String enableJMX = (String) authBindings.getInitApplicationAttribute(SecurityConstants.ENABLE_JMX);
        if (enableJMX != null && "true".equalsIgnoreCase(enableJMX)) {
            Map options = new HashMap();
            options.put(SecurityConstants.MBEAN_SERVER_FOR_CONNECTOR, authBindings.getInitApplicationAttribute(SecurityConstants.MBEAN_SERVER_FOR_CONNECTOR));
            options.put(SecurityConstants.RMI_REGISTRY_HOST, authBindings.getInitApplicationAttribute(SecurityConstants.RMI_REGISTRY_HOST));
            options.put(SecurityConstants.RMI_REGISTRY_PORT, authBindings.getInitApplicationAttribute(SecurityConstants.RMI_REGISTRY_PORT));
            options.put(SecurityConstants.MBEAN_SERVER_FOR_CONNECTOR, authBindings.getInitApplicationAttribute(SecurityConstants.MBEAN_SERVER_FOR_CONNECTOR));
            JMXHelper.enableJMXSecurity(applicationName, options, jguardConf, accessControllerUtils);
        }
    }

    /**
     *
     * @param authorizationConfigurationLocation
     * @param webappHomePath
     * @param authFactoryClass
     * @param accessContext
     * @param applicationName
     * @param authenticationConfigurationLocation
     * @throws java.lang.IllegalStateException
     */
    public static void installAuthorizationAndAuthentication(String authorizationConfigurationLocation, String webappHomePath, String authFactoryClass, AccessContext accessContext, String applicationName, String authenticationConfigurationLocation) throws IllegalStateException {
        Map authorizationSettings = AuthorizationHelper.loadConfiguration(authorizationConfigurationLocation, webappHomePath);
        String authorizationScope = getAuthorizationScope(authorizationSettings);
        AuthenticationBindingsFactory factory;
        try {
            factory = AbstractAuthenticationBindingsFactory.getAuthenticationBindingsFactory(authFactoryClass, authorizationScope);
        } catch (Exception ex) {
            throw new IllegalStateException(ex);
        }
        AuthenticationBindings authBindings = factory.getAuthenticationBindings(accessContext);
        authBindings.setApplicationAttribute(PolicyEnforcementPointOptions.APPLICATION_NAME.getLabel(), applicationName);
        AccessControllerUtils accessControllerUtils = initAuthorization(authBindings, authorizationScope, authorizationSettings, applicationName);
        initAuthentication(authenticationConfigurationLocation, webappHomePath, applicationName, authBindings, accessControllerUtils);
    }

}
