/*
jGuard is a security framework based on top of jaas (java authentication and authorization security).
it is written for web applications, to resolve simply, access control problems.
version $Name$
http://sourceforge.net/projects/jguard/

Copyright (C) 2004  Charles GAY

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


jGuard project home page:
http://sourceforge.net/projects/jguard/

*/
package net.sf.jguard.ext.authentication.loginmodules;

import java.security.NoSuchAlgorithmException;
import java.security.cert.X509Certificate;
import java.util.Map;
import java.util.Set;

import javax.security.auth.Subject;
import javax.security.auth.callback.Callback;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.callback.NameCallback;
import javax.security.auth.callback.PasswordCallback;
import javax.security.auth.callback.UnsupportedCallbackException;
import javax.security.auth.login.LoginException;
import javax.security.auth.spi.LoginModule;

import net.sf.jguard.core.CoreConstants;
import net.sf.jguard.core.authentication.AuthenticationChallengeException;
import net.sf.jguard.core.authentication.callbacks.AsynchronousCallbackException;
import net.sf.jguard.ext.SecurityConstants;
import net.sf.jguard.ext.authentication.callbacks.CertificatesCallback;
import net.sf.jguard.core.util.CryptUtils;
import net.sf.jguard.core.authentication.callbacks.AuthenticationSchemeHandlerCallback;
import net.sf.jguard.core.authentication.credentials.JGuardCredential;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


/**
 * Abstract LoginModule which provides convenient methods for loginmodule
 * involved in grabbing user account informations.
 * @author <a href="mailto:diabolo512@users.sourceforge.net">Charles Gay</a>
 *
 */
public abstract class UserLoginModule implements LoginModule {

	protected  Subject subject;
        protected CallbackHandler callbackHandler;
        protected Map sharedState;
        protected Map options;
        protected boolean debug = false;
        private static final Logger logger = LoggerFactory.getLogger(UserLoginModule.class.getName());
        protected String login = null;
        protected char[] password = null;
	protected boolean skipPasswordCheck;
	protected boolean loginOK = true;
	protected static final String LOGIN_ERROR = "login.error";
        protected static final String USER_INACTIVE = "user.inactive";
        protected Set globalPrincipals;
	protected Set globalPrivateCredentials;
	protected Set globalPublicCredentials;
        protected String authenticationSchemeHandlerName;
	
	

	public void initialize(Subject subj, CallbackHandler cbk, Map sState, Map opts) {

		subject = subj;
		callbackHandler = cbk;
		sharedState=sState;
		options= opts;
                if(options!=null){
                    debug = Boolean.valueOf((String)options.get(CoreConstants.DEBUG)).booleanValue();
                }
	}

	/**
	 * grab login and password from CallbackHandler
	 * and crypt the password with if needed.
	 * @throws LoginException
	 */
	private void grabCredentials() throws LoginException{

       if (callbackHandler == null){
            throw new LoginException("there is no CallbackHandler to authenticate the user");
       }

       Callback[] callbacks = new Callback[4];
       callbacks[0] = new NameCallback("login");
       callbacks[1] = new PasswordCallback("password", false);
       callbacks[2] = new CertificatesCallback();
       callbacks[3] = new AuthenticationSchemeHandlerCallback();

       try {
                    callbackHandler.handle(callbacks);
                    login = ((NameCallback) callbacks[0]).getName();
                    password = ((PasswordCallback) callbacks[1]).getPassword();
                    authenticationSchemeHandlerName = ((AuthenticationSchemeHandlerCallback)callbacks[3]).getAuthenticationSchemeHandlerName();
                    
                    if(login == null || login.equals("")){
                        login = CoreConstants.GUEST;
                        password = CoreConstants.GUEST.toCharArray();
                    }
                    
                    if(CoreConstants.GUEST.equals(login) 
                    	&& CoreConstants.GUEST.toCharArray().equals(password)){
                    	skipPasswordCheck = true;
                    }

                    if(password == null){
                        password="".toCharArray();
                        logger.debug(" password is null");
                    }else{
                        password = CryptUtils.cryptPassword(password);
                         //remove the password from the PasswordCallback
                        ((PasswordCallback)callbacks[1]).clearPassword();
                        if(debug & logger.isDebugEnabled()){
                            logger.debug("login() - usernameFromForm=" + login);
                            logger.debug("login() - passwordFromForm="+ new String(password));
                        }

                    }
                   
                    X509Certificate[] certChainToCheck = ((CertificatesCallback) callbacks[2]).getCertificates();
                    if (certChainToCheck != null) {
                            //in this case (CLIENT-CERT scheme), password is null
                            //and login is given by the Distinguished name of the Subject from the user certificate
                            //there is no need to grab it from an http FORM
                            login = certChainToCheck[0].getSubjectX500Principal().getName();
                            if(debug){
                                    logger.debug(" login used in the certificate ="+login);
                            }
                            skipPasswordCheck=true;
                    }

       } catch (java.io.IOException ioe) {
           throw new LoginException(ioe.toString());
       }catch(AsynchronousCallbackException cnc){
           throw new AuthenticationChallengeException(cnc.getMessage());
       }catch (UnsupportedCallbackException uce) {
           throw new LoginException("Callback error : " + uce.getCallback().toString() +
                   " not available to authenticate the user");
       } catch (NoSuchAlgorithmException e) {
           throw new LoginException("Error encoding password (" + e.getMessage() + ")");
       }
	

    }



	/**
	 * remove Principals and Private/Public credentials from Subject.
	 * @see javax.security.auth.spi.LoginModule#logout()
	 */
	public boolean logout() throws LoginException {
	    if(subject!= null){
	          subject.getPrincipals().clear();
	          subject.getPrivateCredentials().clear();
	          subject.getPublicCredentials().clear();
	    }
	    return true;
	}



	/**
	 * remove Principals and Public/Private Credentials from Subject.
	 * @see javax.security.auth.spi.LoginModule#abort()
	 */
	public boolean abort() throws LoginException {
	    if(subject!= null){
	          subject.getPrincipals().clear();
	          subject.getPrivateCredentials().clear();
	          subject.getPublicCredentials().clear();
	    }
            if(globalPrivateCredentials!=null){
                globalPrivateCredentials.clear();
            }
            if(globalPublicCredentials!=null){
                globalPublicCredentials.clear();
            }
            if(globalPrincipals!=null){
               globalPrincipals.clear();
            }
	    return true;
	}

	public boolean login()throws LoginException{
		skipPasswordCheck = Boolean.valueOf((String)sharedState.get(SecurityConstants.SKIP_PASSWORD_CHECK)).booleanValue();
                grabCredentials();
		
                if(CoreConstants.GUEST.equals(login)){
                    sharedState.put(SecurityConstants.SKIP_PASSWORD_CHECK,"true");
                }
		
		return true;
	}

        /**
	 * add Principals and Public/Private credentials to Subject.
	 * @see javax.security.auth.spi.LoginModule#commit()
	 */
	public boolean commit() throws LoginException {
		if(!loginOK){
			return false;
		}
		Set principals = subject.getPrincipals();
		if(globalPrincipals!=null){
		 principals.addAll(globalPrincipals);
		}
		Set privCredentials = subject.getPrivateCredentials();
		if(globalPrivateCredentials!=null){
		 privCredentials.addAll(globalPrivateCredentials);
		}
		Set pubCredentials = subject.getPublicCredentials();
		if(globalPublicCredentials!= null){
		 pubCredentials.addAll(globalPublicCredentials);
		}
                JGuardCredential cred = new JGuardCredential(CoreConstants.AUTHENTICATION_SCHEME_HANDLER_NAME, authenticationSchemeHandlerName);
                pubCredentials.add(cred);
		return true;
	}

}
